#include <array>
#include <cstdint>
#include <cstdlib>
#include <ostream>
#include <sstream>

#include <pybind11/embed.h>

namespace py = pybind11;

using dword = std::uint32_t;  // legacy

std::array<dword, 1024> g_fpga_config;

struct fpga_config_view
   {
   dword *dwords;
   std::size_t n;
   };

std::ostream &
operator<<(std::ostream &lhs, fpga_config_view const &rhs)
   {
   return lhs << "fpga_config_view@" << &rhs << "{dwords=@" << rhs.dwords << ", n=" << rhs.n << '}';
   }

py::object
py_class_FpgaConfig(py::module const m)
   {
   return
     py::class_<fpga_config_view>(m, "FpgaConfig", py::buffer_protocol())
       .def(py::init<fpga_config_view>())
       .def("__len__", [](fpga_config_view const &cfg)
          {
          return cfg.n;
          })
       .def("__getitem__", [](fpga_config_view const &cfg, std::size_t const i)
          {
          if (i < cfg.n)
             return cfg.dwords[i];

          throw py::index_error();
          })
       .def("__setitem__", [](fpga_config_view &cfg, std::size_t const i, dword const val)
          {
          if (i < cfg.n)
             cfg.dwords[i] = val;
          else
             throw py::index_error();
          })
       .def_buffer([](fpga_config_view &cfg)
          {
          return py::buffer_info(cfg.dwords, static_cast<ssize_t>(cfg.n));
          })
       .def("__str__", [](fpga_config_view const &cfg)
          {
          std::ostringstream ss;
          ss << "FpgaConfig" << '{' << cfg << '}';
          return ss.str();
          });
   }

int
main()
   {
   py::scoped_interpreter interp;
   auto const py_main = py::module::import("__main__");
   auto const py_FpgaConfig = py_class_FpgaConfig(py_main);
   py::eval_file("cppydemo.py", py_main.attr("__dict__"));
   auto const py_process_fpga_config = py_main.attr("process_fpga_config");
   auto const py_fpga_config = py_FpgaConfig(fpga_config_view{ g_fpga_config.data(), g_fpga_config.size() });
   py_process_fpga_config(py_fpga_config);
   return EXIT_SUCCESS;
   }
